"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPlanton = void 0;
const delay_1 = __importDefault(require("delay"));
const Logger_1 = __importDefault(require("../Logger"));
const createEmitter_1 = require("./createEmitter");
const log = Logger_1.default.child({
    namespace: 'createPlanton',
});
const createPlanton = (configuration) => {
    const { getActiveTaskInstructions, } = configuration;
    const events = createEmitter_1.createEmitter();
    const tasks = [];
    for (const inputTask of configuration.tasks) {
        log.debug('registered %s task', inputTask.name);
        const calculateDelay = inputTask.delay || (() => {
            return 1000;
        });
        const concurrency = inputTask.concurrency || 1;
        const task = {
            attemptNumber: 0,
            concurrency,
            exclusive: inputTask.exclusive !== false,
            name: inputTask.name,
            schedule: inputTask.schedule,
        };
        const terminate = (() => {
            let resolveTermination;
            const terminationPromise = new Promise((resolve) => {
                resolveTermination = resolve;
            });
            let active = true;
            (async () => {
                // eslint-disable-next-line no-unmodified-loop-condition
                while (active) {
                    const activeTaskInstructions = await getActiveTaskInstructions(inputTask.name);
                    if (activeTaskInstructions.length >= concurrency) {
                        break;
                    }
                    const taskInstructions = await inputTask.schedule({
                        activeTaskInstructions,
                        limit: concurrency - activeTaskInstructions.length,
                    });
                    if (taskInstructions.length > 0) {
                        task.attemptNumber = 0;
                        for (const taskInstruction of taskInstructions) {
                            events.emit('task', {
                                instruction: taskInstruction,
                                taskName: task.name || '',
                            });
                        }
                    }
                    else if (task.attemptNumber !== undefined) {
                        task.attemptNumber++;
                    }
                    const calculatedDelay = calculateDelay(task.attemptNumber || 0);
                    if (calculatedDelay) {
                        await delay_1.default(calculatedDelay);
                    }
                    if (!active) {
                        break;
                    }
                }
                if (resolveTermination) {
                    resolveTermination();
                }
            })();
            return () => {
                active = false;
                return terminationPromise;
            };
        })();
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        task.terminate = terminate;
        tasks.push(task);
    }
    return {
        events,
        terminate: async () => {
            await Promise.all(tasks.map((task) => {
                return task.terminate();
            }));
        },
    };
};
exports.createPlanton = createPlanton;
